/*
   "romcrc.c", Version 2.01 vom 29.4.1999, (C) Volker Ernst 1999
   Simple c source-file to calculate and write the rom-crc-value.
   Part of the source-files for HLB, the harmless little box.
   Free for non-commercial usage, written by xorrox@gmx.net.
*/

/*---------*/
/* DEFINES */
/*---------*/

#define ROM_CHKSIZE (0x4000)

/*----------*/
/* INCLUDES */
/*----------*/

#include <stdlib.h>
#include <stdio.h>

/*------------*/
/* PROTOTYPES */
/*------------*/

int main (int argc, char *argv[]);

/*-----------*/
/* VARIABLEN */
/*-----------*/

static unsigned char rom_buffer[ROM_CHKSIZE];
static unsigned long crc_buffer[256];

/*==========*/
/* PROGRAMM */
/*==========*/

int main (int argc, char *argv[])
{
  unsigned short n,m,ix,x;
  unsigned long chksum;
  FILE *fp;

  if (argc != 2)                                      /* CHECK_ARGUMENTS */
  {
    fputs ("romcrc: usage: romcrc [Filename]\n", stderr);
    return (1);
  }

  fp = fopen (argv[1],"r+");                          /* OPEN_ROM-IMAGE */
  if (fp==NULL)
  {
    fputs ("romcrc: error: can't open rom-image \"", stderr);
    fputs (argv[1], stderr);
    fputs ("\" !\n", stderr);
    return (2);
  }

  n = fread (rom_buffer, 1, ROM_CHKSIZE-4, fp);       /* READ_ROM-IMAGE */
  if (n != ROM_CHKSIZE-4)
  {
    fputs ("romcrc: error: can't read rom-image \"", stderr);
    fputs (argv[1], stderr);
    fputs ("\" !\n", stderr);
    fclose (fp);
    return (3);
  }

  for (n=0; n<256; n++)                               /* BUILD CRC-TABLE */
  {
    chksum = 0L;
    ix = n;
    for (m=0; m<8; m++)
    {
      x = (chksum^ix) & 1;
      chksum = chksum>>1;
      ix = ix>>1;
      if (x) chksum=chksum^0xEDB88320;
    }
    crc_buffer[n] = chksum;
  }

  chksum = 0xFFFFFFFFL;                               /* CALC_CRC-VALUE */
  for (n=0; n<ROM_CHKSIZE-4; n++)
  {
    m = (chksum^rom_buffer[n]) & 0xFF;
    chksum = (chksum>>8) ^ crc_buffer[m];
  }
  rom_buffer[ROM_CHKSIZE-4] = chksum >>  0;
  rom_buffer[ROM_CHKSIZE-3] = chksum >>  8;
  rom_buffer[ROM_CHKSIZE-2] = chksum >> 16;
  rom_buffer[ROM_CHKSIZE-1] = chksum >> 24;

  fseek (fp, 0L, SEEK_CUR);                           /* WRITE_ROM-IMAGE */
  n = fwrite (rom_buffer+ROM_CHKSIZE-4, 1, 4, fp);
  if (n != 4)
  {
    fputs ("romcrc: error: can't write rom-image \"", stderr);
    fputs (argv[1], stderr);
    fputs ("\" !\n", stderr);
    fclose (fp);
    return (4);
  }

  fclose (fp);                                        /* CLOSE_ROM-IMAGE */
  return 0;
}


/*** EOF ***/
