/* Include for Gnu Multi Precision-Lib */
#include "../gmp-2.0.2/gmp.h"

/* #define DEBUG */
#define RABINMILLERCYCLES	25

#define TRUE 1
#define FALSE 0

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <assert.h>

extern void HLBCryptRandomStart(void);
extern void HLBCryptRandomGet(unsigned char randombytes[16]);
extern int  HLBCryptRandomGetLarge(mpz_t rndzahl, int bits);

#ifdef DEBUG
void
printBigNumber(
mpz_t number) 	/* IN number to be diplayed */
{
	char *cptr;
	
	cptr = (char *)malloc(mpz_sizeinbase(number, 10) + 5); 
	mpz_get_str(cptr, 10, number);
	printf("%s\n", cptr);
        free(cptr); 	
}
#endif DEBUG


int		/* false = something went wrong */
HLBCryptRsaGen(
mpz_t n,	/* OUT public part (also used by private) */
mpz_t e,	/* OUT public part */
mpz_t d,	/* OUT private part */
int bits)
{
	mpz_t p, q, store, result;
	int i, test;

	mpz_init(p);
#ifdef DEBUG	
	printf("searching prime by testing randoms:");
#endif	
	/* generate a new Prime */
	do {	
		test = HLBCryptRandomGetLarge(p, bits);
		assert(test);
		mpz_setbit(p, 0);
#ifdef DEBUG
		printf("."); fflush(stdout);
#endif		
	} while (mpz_probab_prime_p(p, RABINMILLERCYCLES) == 0);
#ifdef DEBUG
	printf("\np=");
	printBigNumber(p);
#endif 

	mpz_init(q);
#ifdef DEBUG	
	printf("searching prime by testing randoms:");
#endif	
	/* generate a new Prime */
	do {	
		test = HLBCryptRandomGetLarge(q, bits);
		assert(test);
		mpz_setbit(q, 0);
#ifdef DEBUG
		printf("."); fflush(stdout);
#endif		
	} while (mpz_probab_prime_p(q, RABINMILLERCYCLES) == 0);
#ifdef DEBUG
	printf("\nq=");
	printBigNumber(q);
#endif 

	mpz_init(n);
	mpz_mul(n, p, q);
#ifdef DEBUG
	printf("\nn=");
	printBigNumber(n);
#endif 

	mpz_init(store);
	mpz_sub_ui(p, p, 1);
	mpz_sub_ui(q, q, 1);
	mpz_mul(store, p, q);	/* store = (p-1)*(q-1) */
#ifdef DEBUG
	printf("\n(p-1)*(q-1)=");
	printBigNumber(store);
#endif 

	mpz_init(e);
	mpz_init(result);
again:
	/* find random e that relativly prime to (p-1)*(q-1) */
#ifdef DEBUG
	printf("\nsearching random that has gcd=1 with (p-1)*(q-1):");
#endif	
	do {	
		test = HLBCryptRandomGetLarge(e, bits);
		assert(test);
#ifdef DEBUG
		printf("."); fflush(stdout); 
#endif		
		if (mpz_cmp(e, store) > 0)	continue;	/* 0 < e < (p-1)*(q-1) war nicht erfuellt */		
		mpz_gcd(result, e, store);
	} while (mpz_cmp_ui(result, 1) != 0);
#ifdef DEBUG
	printf("\ne=");
	printBigNumber(e);
#endif 

	mpz_init(d);
	i = mpz_invert(d, e, store);
#ifdef DEBUG
	printf("\nd=");
	printBigNumber(d);
#endif 
	if (i == 0) 			goto again;	/* we have no inverse modulo */
	if (mpz_cmp_ui(d, 1) <= 0) 	goto again;	/* d is negative */
	mpz_gcd(result, d, n);
        if (mpz_cmp_ui(result, 1) != 0) goto again;	/* d is not relative prime to n */


	mpz_clear(p);
	mpz_clear(q);
	mpz_clear(store); 
	mpz_clear(result);
	return(TRUE);
}

int		/* false = something went wrong */
HLBCryptRsaEncrypt(
mpz_t n,	/* IN public part */
mpz_t e,	/* IN public part */
mpz_t m,	/* IN Message to encrypt (0 < m < n) */
mpz_t c)	/* OUT encrypted Message */
{
	mpz_init(c);
	mpz_powm(c, m, e, n); 
#ifdef DEBUG
	printf("\nc=");
	printBigNumber(c);
#endif 	
	return(TRUE);
}

int		/* false = something went wrong */
HLBCryptRsaDecrypt(
mpz_t n,	/* IN public part */
mpz_t d,	/* IN private part */
mpz_t c,	/* IN Message to decrypt */
mpz_t m)	/* OUT decrypted Message */
{
	mpz_init(m);
	mpz_powm(m, c, d, n); 
#ifdef DEBUG
	printf("\nm=");
	printBigNumber(m);
#endif 	
	return(TRUE);
}


#ifdef DEBUG
int
main(void) {
	mpz_t n, e, d, c, m, test;
	
	HLBCryptRandomStart();

	HLBCryptRsaGen(n, e, d, 2048);
	mpz_init(test);
	mpz_init(m);
	mpz_set_str(m, "123456789012345678901234567890", 10);	
	HLBCryptRsaEncrypt(n, e, m, c);
	HLBCryptRsaDecrypt(n, d, c, test);
	return(0);
}
#endif
