/* hlbmain.c */


#include "hlbports.h"
#include "hlbios.h"

#include <stdio.h>
#include <malloc.h>

extern void pcon_puts (const unsigned char *ptr);
#include "../crypto/gmp-2.0.2/gmp.h"
extern void HLBCryptRandomStart(void);
extern void HLBCryptRandomGet(unsigned char randombytes[16]);
extern int  HLBCryptRandomGetLarge(mpz_t rndzahl, int bits);
extern int  HLBCryptElgamalGen(mpz_t p, mpz_t g, mpz_t y, mpz_t x, int bits);
extern int  HLBCryptElgamalEncrypt(mpz_t p, mpz_t g, mpz_t y, mpz_t M, mpz_t a, mpz_t b, int bits);
extern int  HLBCryptElgamalDecrypt(mpz_t p, mpz_t g, mpz_t x, mpz_t a, mpz_t b, mpz_t M);

#define GSMTEST

#ifdef GSMTEST
#include "../codecs/gsm/gsm.h"

gsm handle;
gsm_frame buf;
gsm_signal sample[160];

#include "test.c"
#endif


void
printBig(mpz_t number)   /* IN number to be diplayed */
{
        char *cptr;
        
	cptr = (char *)malloc(mpz_sizeinbase(number, 10) + 5);
   	mpz_get_str(cptr, 10, number);
   	pcon_puts (cptr);
   	pcon_puts ("\r\n");
        free(cptr);
}
                                        



/*--------*/

int main (void);

void pcon_hexdump (unsigned long Start, unsigned long Len, unsigned char Size);
void pcon_puts (const unsigned char *Text);
void pcon_putlong  (unsigned long  Data);
void pcon_putword  (unsigned short Data);
void pcon_putbyte  (unsigned char  Data);
void pcon_putdigit (unsigned char  Data);

/*--------*/

int main (void)
{
#ifdef GSMTEST
        unsigned char chr;
        int i, j, k, ganzz;
        unsigned char *cptr;
        gsm_signal *ganz;
#endif

  char *testc;

	mpz_t p, g, y, x, a, b, M, test;

/* fixme */
/* debug memory overwrites by realloc function */
/*
	mpz_init(p);
	mpz_init(g);
	mpz_set_str(p, "17076595864968010063959113649807603542250075059814893511311470186566667419145959445173375438207512", 10);
        j = p->_mp_size;
        for (i = 0; i < j; i++) {
        	pcon_putlong(p->_mp_d[i]);
        	pcon_puts("\r\n");
        }
	printBig(p); pcon_puts("\r\n17076595864968010063959113649807603542250075059814893511311470186566667419145959445173375438207512\r\n");
	mpz_set_str(g, "-71031537616988171965293723577903057129971490463300314885805156419098070549057402071248121020446", 10);
	j = -g->_mp_size;
        for (i = 0; i < j; i++) {
        	pcon_putlong(g->_mp_d[i]);
        	pcon_puts("\r\n");
        }
	printBig(g); pcon_puts("\r\n-71031537616988171965293723577903057129971490463300314885805156419098070549057402071248121020446\r\n");
	mpz_sub(p, g, p);
	printBig(p);
        pcon_puts("\r\n-17147627402584998235924407373385506599380046550278193826197275342985765489695016847244623559227958\r\n");
*/


	
	pcon_puts ("ElGamal generate public/private Keypairs\r\n\r\n");
	HLBCryptRandomStart();

	malloc(1000);
	mpz_init(p);
	malloc(1000);
	mpz_init(g);	
	malloc(1000);
	mpz_init(x);
	malloc(1000);
	mpz_init(y);	
	malloc(1000);
	mpz_init(a);
	malloc(1000);
	mpz_init(b);	
	malloc(1000);
	mpz_init(M);
	malloc(1000);
	mpz_init(test);

	HLBCryptElgamalGen(p, g, y, x, 512);
	pcon_puts ("\r\np=");
	printBig(p);
	pcon_puts ("\r\ng=");
	printBig(g);	
	pcon_puts ("\r\ny=");
	printBig(y);	
	pcon_puts ("\r\nx=");
	printBig(x);
	mpz_init(test);
	mpz_init(M);
	pcon_puts ("\r\nencrypt\r\n");
	mpz_set_str(M, "123456789012345678901234567890", 10);	
	pcon_puts ("\r\nM=");
	printBig(M);
	HLBCryptElgamalEncrypt(p, g, y, M, a, b, 512);
	pcon_puts ("\r\na=");
	printBig(a);
	pcon_puts ("\r\nb=");
	printBig(b);
	pcon_puts ("\r\ndecrypt\r\n");
	HLBCryptElgamalDecrypt(p, g, x, a, b, test);
	pcon_puts ("\r\nM=");
	printBig(test);


  pcon_puts ("start gsm\r\n");

#ifdef GSMTEST

	handle = gsm_create();
        k = 1;
	gsm_option(handle, GSM_OPT_FAST, &k);
        k = 1;
	gsm_option(handle, GSM_OPT_LTP_CUT, &k);


	ganz = (gsm_signal *)malloc(160*622*2);

	ganzz = 0;
        cptr = &gsmtestframes[0];
        for (i = 0; i < 620; i++) {
                gsm_decode(handle, cptr, sample);
                for (j = 0; j < 160; j++) {
                	ganz[ganzz] = (sample[j] & 0xfffc);
                	ganzz++;
                }
                cptr += 33;
        };

	for (j = 0; j < 3; j++) {
		padda_xchg(ganz, 160*620, 1);    
        };
        gsm_destroy(handle);

#endif

  pcon_puts ("\r\nDas ist C-Code :-)\r\n");

  while (42)
  {
  }
  return 0;
}

/*--------*/

void pcon_hexdump (unsigned long adr, unsigned long len, unsigned char size)
{
  unsigned short n,m;
  unsigned long ptr,anz;

  pcon_puts ("Memory-Dump von ");
  pcon_putlong (adr);
  pcon_puts (" bis ");
  pcon_putlong (adr+len-1);
  pcon_puts ("\r\n");

  for (m=1;m<=size;m<<=1)
  {
    ptr = adr;
    anz = len;
    while (anz)
    {
      pcon_putlong (ptr);
      pcon_puts ("  ");
      for (n=0;n<16;n+=m)
      {
        pcon_putch (' ');
        if (m==1) 
          pcon_putbyte (*((unsigned char *)ptr));
        else if (m==2)
          pcon_putword (*((unsigned short *)ptr));
        else if (m==4)
          pcon_putlong (*((unsigned long *)ptr));
        ptr += m;
        anz -= m;
      }
      pcon_puts ("\r\n");
    }
  }
}

void pcon_putlong (unsigned long data)
{
  pcon_putword (data>>16);
  pcon_putword (data);
}

void pcon_putword (unsigned short data)
{
  pcon_putbyte (data>>8);
  pcon_putbyte (data);
}

void pcon_putbyte (unsigned char data)
{
  pcon_putdigit (data>>4);
  pcon_putdigit (data);
}

void pcon_putdigit (unsigned char data)
{
  data = (data & 0x0F) + '0';
  if (data > '9') data += 7;
  pcon_putch (data);
}

void pcon_puts (const unsigned char *ptr)
{
  while (*ptr)
    pcon_putch (*ptr++);
}


/*** EOF ***/
