/*
   "syscrc.c", Version 2.01 vom 29.4.1999, (C) Volker Ernst 1999
   Simple c source-file to calculate and write the sys-crc-value.
   Part of the source-files for HLB, the harmless little box.
   Free for non-commercial usage, written by xorrox@gmx.net.
*/

/*---------*/
/* DEFINES */
/*---------*/

#define SYS_CHKSIZE (0x0001C000L)

/*----------*/
/* INCLUDES */
/*----------*/

#include <stdlib.h>
#include <stdio.h>

/*------------*/
/* PROTOTYPES */
/*------------*/

int main (int argc, char *argv[]);

/*-----------*/
/* VARIABLEN */
/*-----------*/

static unsigned char sys_buffer[SYS_CHKSIZE];
static unsigned long crc_buffer[256];

/*==========*/
/* PROGRAMM */
/*==========*/

int main (int argc, char *argv[])
{
  unsigned short n,m,ix,x;
  unsigned long chksum,i;
  FILE *fp;

  if (argc != 2)                                      /* CHECK_ARGUMENTS */
  {
    fputs ("syscrc: usage: syscrc [Filename]\n", stderr);
    return (1);
  }

  fp = fopen (argv[1],"r+");                          /* OPEN_SYS-IMAGE */
  if (fp==NULL)
  {
    fputs ("syscrc: error: can't open sys-image \"", stderr);
    fputs (argv[1], stderr);
    fputs ("\" !\n", stderr);
    return (2);
  }

  i = fread (sys_buffer, 1, SYS_CHKSIZE-4, fp);       /* READ_SYS-IMAGE */
  if (i < 0x0100)
  {
    fputs ("syscrc: error: can't read sys-image \"", stderr);
    fputs (argv[1], stderr);
    fputs ("\" !\n", stderr);
    fclose (fp);
    return (3);
  }

  for (; i<SYS_CHKSIZE-4; i++)                        /* FILL_SYS-IMAGE */
    sys_buffer[i] = 0xFF;

  for (n=0; n<256; n++)                               /* BUILD CRC-TABLE */
  {
    chksum = 0L;
    ix = n;
    for (m=0; m<8; m++)
    {
      x = (chksum^ix) & 1;
      chksum = chksum>>1;
      ix = ix>>1;
      if (x) chksum=chksum^0xEDB88320;
    }
    crc_buffer[n] = chksum;
  }

  chksum = 0xFFFFFFFFL;                               /* CALC_CRC-VALUE */
  for (i=0; i<SYS_CHKSIZE-4; i++)
  {
    m = (chksum^sys_buffer[i]) & 0xFF;
    chksum = (chksum>>8) ^ crc_buffer[m];
  }
  sys_buffer[SYS_CHKSIZE-4] = chksum >>  0;
  sys_buffer[SYS_CHKSIZE-3] = chksum >>  8;
  sys_buffer[SYS_CHKSIZE-2] = chksum >> 16;
  sys_buffer[SYS_CHKSIZE-1] = chksum >> 24;

  fseek (fp, 0L, SEEK_SET);                           /* WRITE_SYS-IMAGE */
  i = fwrite (sys_buffer, 1, SYS_CHKSIZE, fp);
  if (i != SYS_CHKSIZE)
  {
    fputs ("syscrc: error: can't write sys-image \"", stderr);
    fputs (argv[1], stderr);
    fputs ("\" !\n", stderr);
    fclose (fp);
    return (4);
  }

  fclose (fp);                                        /* CLOSE_SYS-IMAGE */
  return 0;
}


/*** EOF ***/
